from django.test import TestCase
from mock import patch

from testproject.forms import BleachForm


class TestBleachField(TestCase):

    def test_empty(self):
        """ Test no data """
        form = BleachForm(data={})
        form.is_valid()
        self.assertEqual(form.fields['no_tags'].to_python(None), None)
        self.assertEqual(form.fields['no_tags'].to_python(''), '')
        self.assertEqual(form.fields['no_tags'].to_python([]), [])
        self.assertEqual(form.fields['no_tags'].to_python({}), {})

    def test_bleaching(self):
        """ Test values are bleached """
        test_data = {
            'no_tags': "<h1>Heading</h1>",
            'no_strip': "<h1>Heading</h1>",
            'bleach_strip': "<!-- script here -->"
                            "<script>alert(\"Hello World\")</script>",
            'bleach_attrs': "<a href=\"https://www.google.com\" "
                            "target=\"_blank\">google.com</a>",
            'bleach_styles': "<li style=\"color: white\">item</li>"
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(
            form.cleaned_data['no_tags'], 'Heading'
        )
        self.assertEqual(
            form.cleaned_data['no_strip'],
            '&lt;h1&gt;Heading&lt;/h1&gt;'
        )
        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            'alert("Hello World")'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            '<a href="https://www.google.com">google.com</a>'
        )
        self.assertNotEqual(
            form.cleaned_data['bleach_styles'],
            test_data['bleach_styles']
        )

    def test_tags(self):
        """ Test allowed tags are rendered"""
        test_data = {
            'no_tags': "<p>No tags here</p>",
            'no_strip': "No tags here",
            'bleach_strip': "<ul><li>one</li><li>two</li></ul>",
            'bleach_attrs': "<a href=\"https://www.google.com\" "
                            "title=\"Google\">google.com</a>",
            'bleach_styles': "<li style=\"color: white;\">item</li>"
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(form.cleaned_data['no_tags'], "No tags here")
        self.assertEqual(form.cleaned_data['no_strip'], "No tags here")

        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            test_data['bleach_strip']
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            test_data['bleach_attrs']
        )
        self.assertEqual(
            form.cleaned_data['bleach_styles'],
            test_data['bleach_styles']
        )

    def test_attrs(self):
        """ Test allowed attributes are rendered """
        list_html = "<ul class=\"our-list\">" \
                    "<li class=\"list-item\">one</li>" \
                    "<li>two</li>" \
                    "</ul>"
        test_data = {
            'no_strip': "",
            'no_tags': list_html,
            'bleach_strip': list_html,
            'bleach_attrs': list_html,
            'bleach_styles': list_html
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(form.cleaned_data['no_tags'], 'onetwo')

        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            '<ul><li>one</li><li>two</li></ul>'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            test_data['bleach_strip']
        )
        self.assertEqual(
            form.cleaned_data['bleach_styles'],
            '<ul><li>one</li><li>two</li></ul>'
        )

    @patch('django_bleach.forms.settings',
           BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget')
    def test_custom_widget(self, settings):

        self.assertEqual(
            settings.BLEACH_DEFAULT_WIDGET,
            'testproject.forms.CustomBleachWidget'
        )
        test_data = {
            'no_tags': "<h1>Heading</h1>",
            'no_strip': "<h1>Heading</h1>",
            'bleach_strip': "<!-- script here -->"
                            "<script>alert(\"Hello World\")</script>",
            'bleach_attrs': "<a href=\"http://www.google.com\" "
                            "target=\"_blank\">google.com</a>",
            'bleach_styles': "<li style=\"color: white\">item</li>"
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(
            form.cleaned_data['no_tags'], 'Heading'
        )
        self.assertEqual(
            form.cleaned_data['no_strip'],
            '&lt;h1&gt;Heading&lt;/h1&gt;'
        )
        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            'alert("Hello World")'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            '<a>google.com</a>'
        )
        self.assertNotEqual(
            form.cleaned_data['bleach_styles'],
            test_data['bleach_styles']
        )

    @patch('django_bleach.forms.settings',
           BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget')
    def test_custom_widget2(self, settings):
        self.assertEqual(
            settings.BLEACH_DEFAULT_WIDGET,
            'testproject.forms.CustomBleachWidget'
        )
        test_data = {
            'no_tags': "<h1>Heading</h1>",
            'no_strip': "<h1>Heading</h1>",
            'bleach_strip': "<!-- script here -->"
                            "<script>alert(\"Hello World\")</script>",
            'bleach_attrs': "<a href=\"https://www.google.com\" "
                            "target=\"_blank\">google.com</a>",
            'bleach_styles': "<li style=\"color: white\">item</li>"
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(
            form.cleaned_data['no_tags'], 'Heading'
        )
        self.assertEqual(
            form.cleaned_data['no_strip'],
            '&lt;h1&gt;Heading&lt;/h1&gt;'
        )
        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            'alert("Hello World")'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            '<a href="https://www.google.com">google.com</a>'
        )
        self.assertNotEqual(
            form.cleaned_data['bleach_styles'],
            test_data['bleach_styles']
        )



from django import forms
from django.test import TestCase, override_settings
from django_bleach import forms as bleach_forms
from .test_models import BleachContent


class BleachContentModelForm(forms.ModelForm):
    class Meta:
        model = BleachContent
        fields = '__all__'


class TestModelFormField(TestCase):
    @override_settings(BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget')
    def setUp(self):
        model_form = BleachContentModelForm()
        self.form_field = model_form.fields['content']
        self.choice_form_field = model_form.fields['choice']
        self.model_field = BleachContent()._meta.get_field('content')
        self.default_widget_class = bleach_forms.get_default_widget()

    def test_formfield_type(self):
        """ Check content's form field is instance of BleachField
        """
        self.assertIsInstance(self.form_field, bleach_forms.BleachField)

    def test_custom_widget(self):
        """ Check content form field's widget is instance of default widget
        """
        self.assertIsInstance(self.form_field.widget, self.default_widget_class)

    def test_same_allowed_args(self):
        """ Check model and form's allowed arguments (tags, attributes, ...) are same
        """
        form_allowed_args: dict = self.form_field.bleach_options
        model_allowed_args: dict = self.model_field.bleach_kwargs

        self.assertEqual(model_allowed_args, form_allowed_args)

    def test_with_choices(self):
        """ Check if choices specified, use TextField's default widget (Select).
        """
        form_field_widget = self.choice_form_field.widget.__class__
        self.assertEqual(form_field_widget, forms.widgets.Select)



from django.db import models

from bleach import clean

from . import forms
from .utils import get_bleach_default_options


class BleachField(models.TextField):
    def __init__(self, allowed_tags=None, allowed_attributes=None,
                 allowed_styles=None, allowed_protocols=None,
                 strip_tags=None, strip_comments=None, *args, **kwargs):

        super(BleachField, self).__init__(*args, **kwargs)

        self.bleach_kwargs = get_bleach_default_options()

        if allowed_tags:
            self.bleach_kwargs["tags"] = allowed_tags
        if allowed_attributes:
            self.bleach_kwargs["attributes"] = allowed_attributes
        if allowed_styles:
            self.bleach_kwargs["styles"] = allowed_styles
        if allowed_protocols:
            self.bleach_kwargs["protocols"] = allowed_protocols
        if strip_tags:
            self.bleach_kwargs["strip"] = strip_tags
        if strip_comments:
            self.bleach_kwargs["strip_comments"] = strip_comments

    def formfield(self, **kwargs):
        """ Makes the field for a ModelForm """

        # If field doesn't have any choice return BleachField
        if not self.choices:
            return forms.BleachField(
                label=self.verbose_name,
                max_length=self.max_length,
                allowed_tags=self.bleach_kwargs.get("tags"),
                allowed_attributes=self.bleach_kwargs.get("attributes"),
                allowed_styles=self.bleach_kwargs.get("styles"),
                allowed_protocols=self.bleach_kwargs.get("protocols"),
                strip_tags=self.bleach_kwargs.get("strip"),
                strip_comments=self.bleach_kwargs.get("strip_comments"),
            )

        return super(BleachField, self).formfield(**kwargs)

    def pre_save(self, model_instance, add):
        data = getattr(model_instance, self.attname)
        if data:
            clean_value = clean(
                data,
                **self.bleach_kwargs
            )
            setattr(model_instance, self.attname, clean_value)
            return clean_value
        return data



# -*- coding: utf-8 -*-
from django.core.exceptions import ImproperlyConfigured
from django.forms import Textarea
from django.test import TestCase, override_settings
from mock import patch

from django_bleach.forms import get_default_widget
from django_bleach.utils import get_bleach_default_options

from testproject.constants import (
    ALLOWED_ATTRIBUTES, ALLOWED_PROTOCOLS,
    ALLOWED_STYLES, ALLOWED_TAGS,
)
from testproject.forms import CustomBleachWidget


class TestBleachOptions(TestCase):

    @patch('django_bleach.utils.settings',
           BLEACH_ALLOWED_ATTRIBUTES=ALLOWED_ATTRIBUTES)
    def test_custom_attrs(self, settings):
        bleach_args = get_bleach_default_options()
        self.assertEqual(bleach_args['attributes'], ALLOWED_ATTRIBUTES)

    @patch('django_bleach.utils.settings',
           BLEACH_ALLOWED_PROTOCOLS=ALLOWED_PROTOCOLS)
    def test_custom_proto(self, settings):
        bleach_args = get_bleach_default_options()
        self.assertEqual(bleach_args['protocols'], ALLOWED_PROTOCOLS)

    @patch('django_bleach.utils.settings',
           BLEACH_ALLOWED_STYLES=ALLOWED_STYLES)
    def test_custom_styles(self, settings):
        bleach_args = get_bleach_default_options()
        self.assertEqual(bleach_args['styles'], ALLOWED_STYLES)

    @patch('django_bleach.utils.settings', BLEACH_ALLOWED_TAGS=ALLOWED_TAGS)
    def test_custom_tags(self, settings):
        bleach_args = get_bleach_default_options()
        self.assertEqual(bleach_args['tags'], ALLOWED_TAGS)

    @patch('django_bleach.utils.settings', BLEACH_STRIP_TAGS=True)
    def test_strip_tags(self, settings):
        bleach_args = get_bleach_default_options()
        self.assertEqual(bleach_args['strip'], True)

    @patch('django_bleach.utils.settings', BLEACH_STRIP_COMMENTS=True)
    def test_strip_comments(self, settings):
        bleach_args = get_bleach_default_options()
        self.assertEqual(bleach_args['strip_comments'], True)


class TestDefaultWidget(TestCase):
    """ Test form field widgets """

    @override_settings(BLEACH_DEFAULT_WIDGET='django.forms.widgets.Textarea')
    def test_default_widget(self):
        self.assertEqual(get_default_widget(), Textarea)

    @patch('django_bleach.forms.settings',
           BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget')
    def test_custom_widget(self, settings):
        self.assertEqual(get_default_widget(), CustomBleachWidget)

    @patch('django_bleach.forms.settings',
           BLEACH_DEFAULT_WIDGET='testproject.forms.NoneExistentWidget')
    def test_attribute_err(self, settings):
        with self.assertRaises(ImproperlyConfigured):
            get_default_widget()

    @patch('django_bleach.forms.settings',
           BLEACH_DEFAULT_WIDGET='testproject.forms2.CustomBleachWidget')
    def test_import_err(self, settings):
        with self.assertRaises(ImproperlyConfigured):
            get_default_widget()



from django.conf import settings


def get_bleach_default_options():
    bleach_args = {}
    bleach_settings = {
        "BLEACH_ALLOWED_TAGS": "tags",
        "BLEACH_ALLOWED_ATTRIBUTES": "attributes",
        "BLEACH_ALLOWED_STYLES": "styles",
        "BLEACH_STRIP_TAGS": "strip",
        "BLEACH_STRIP_COMMENTS": "strip_comments",
        "BLEACH_ALLOWED_PROTOCOLS": "protocols"
    }

    for setting, kwarg in bleach_settings.items():
        if hasattr(settings, setting):
            bleach_args[kwarg] = getattr(settings, setting)

    return bleach_args



from django.db import models
from django.test import TestCase

from django_bleach.models import BleachField
from testproject.constants import (
    ALLOWED_ATTRIBUTES, ALLOWED_PROTOCOLS,
    ALLOWED_STYLES, ALLOWED_TAGS
)


class BleachContent(models.Model):
    """ Bleach test model"""
    CHOICES = (
        ('f', 'first choice'),
        ('s', 'second choice')
    )
    content = BleachField(
        allowed_attributes=ALLOWED_ATTRIBUTES,
        allowed_protocols=ALLOWED_PROTOCOLS,
        allowed_styles=ALLOWED_STYLES,
        allowed_tags=ALLOWED_TAGS,
        strip_comments=True,
        strip_tags=True
    )
    choice = BleachField(choices=CHOICES)


class TestBleachModelField(TestCase):
    """ Test model field """

    def test_bleaching(self):
        """ Test values are bleached """
        test_data = {
            'no_tags': "<h1>Heading</h1>",
            'no_strip': "<h1>Heading</h1>",
            'bleach_strip': """<script>alert("Hello World")</script>""",
            'bleach_attrs': "<a href=\"http://www.google.com\" "
                            "target=\"_blank\">google.com</a>",
            'bleach_styles': "<li style=\"color: white\">item</li>",
            'bleach_comment': "<!-- this is a comment -->",
        }
        expected_values = {
            'no_tags': "Heading",
            'no_strip': "Heading",
            'bleach_strip': """alert("Hello World")""",
            'bleach_attrs':
                "<a>google.com</a>",
            'bleach_styles': "<li style=\"color: white;\">item</li>",
            'bleach_comment': ""
        }

        for key, value in test_data.items():
            obj = BleachContent.objects.create(content=value)
            self.assertEqual(obj.content, expected_values[key])


class BleachNullableContent(models.Model):
    """ Bleach test model"""
    content = BleachField(
        allowed_attributes=ALLOWED_ATTRIBUTES,
        allowed_protocols=ALLOWED_PROTOCOLS,
        allowed_styles=ALLOWED_STYLES,
        allowed_tags=ALLOWED_TAGS,
        strip_comments=True,
        strip_tags=True,
        blank=True,
        null=True
    )


class TestBleachNullableModelField(TestCase):
    """ Test model field """

    def test_bleaching(self):
        """ Test values are bleached """
        test_data = {
            'none': None,
            'empty': "",
            'whitespaces': "   ",
            'linebreak': "\n",
        }
        expected_values = {
            'none': None,
            'empty': "",
            'whitespaces': "   ",
            'linebreak': "\n",
        }

        for key, value in test_data.items():
            obj = BleachNullableContent.objects.create(content=value)
            self.assertEqual(obj.content, expected_values[key])
